@@ -1,4 +1,4 @@
-//! SorteryLib is a fast file-sorting library based on the Sortery command-line file sorter.
+//! SorteryLib is a fast, cross-platform file-sorting library based on the Sortery command-line file sorter.
 //! 
 //! Basic usage example:
 //! 
@@ -33,25 +33,27 @@
 //! 
 //! You can find more detailed descriptions of the fields on the [`Sorter`] page.
 
-#![cfg(target_os = "linux")]
+mod errors;
 pub mod structs;
 
 use chrono::{DateTime, TimeZone, Utc, Local};
-use std::{fs, path::Path, time::UNIX_EPOCH};
+use errors::*;
+use filetime::FileTime;
+use std::{fs, path::Path};
 use structs::*;
 use walkdir::WalkDir;
 
 /// Includes all the stuff needed for basic operations, in one neat module.
 #[allow(unused_imports)]
 pub mod prelude {
-    pub use crate::Sorter;
+    pub use crate::{FromJson, Sorter};
     pub use crate::structs::{File, Join};
 }
 
 /// Tests. Each test is named after the function or struct it tests, prefixed with `test_`.
 #[cfg(test)]
 mod tests {
-    use crate::Sorter;
+    use crate::{FromJson, Sorter};
     use std::{env, fs, path::Path};
     use super::structs::*;
 
@@ -90,6 +92,11 @@ mod tests {
     }
 }
 
+/// The trait used by [`Sorter`] to either parse a JSON string or a JSON file
+pub trait FromJson<T> {
+    fn from_json(json: T, source: File, target: File) -> Sorter;
+}
+
 /// The sorter struct that sorts the files. There are two ways to create an instance
 /// of [`Sorter`]: passing the individual fields, and using [`Sorter::from_json`].
 #[derive(Debug, PartialEq)]
@@ -123,52 +130,6 @@ pub struct Sorter {
 }
 impl Sorter {
 
-    // Class functions
-
-    /// Return a new [`Sorter`] instance, created using the configuration data in
-    /// a JSON string. The source and target directories must be passed to [`Sorter::from_json`]
-    /// as well as the json string. For example:
-    /// 
-    /// ```ignore
-    /// use sorterylib::prelude::*;
-    /// use std::{fs, path::Path};
-    /// 
-    /// fn main() {
-    /// 
-    ///     // The path to the JSON file
-    ///     let json_path = Path::new("test.json");
-    /// 
-    ///     // Load the JSON string from the JSON file
-    ///     let json_string = fs::read_to_string(json_path).expect("Failed to read file");
-    /// 
-    ///     // Create a Sorter instance based on the json file
-    ///     let sorter = Sorter::from_json(
-    ///         json_string,
-    ///         File::from("/path/to/source/dir/"), // The source directory
-    ///         File::from("/path/to/target/dir/") // The target directory
-    ///     );
-    /// }
-    /// ```
-    /// 
-    /// In future versions, a JSON file will be able to be passed instead of a JSON [`String`].
-    pub fn from_json(json_string: String, source: File, target: File) -> Sorter {
-
-        // Get the data from the JSON string
-        let data = ConfigData::from_json(&json_string);
-
-        Sorter {
-            source: source,
-            target: target,
-            date_format: data.date_format,
-            date_type: data.date_type,
-            preserve_name: data.preserve_name,
-            exclude_type: data.exclude_type,
-            only_type: data.only_type
-        }
-    }
-
-    // Methods
-
     /// Return a [`DateTime`] instance representing the creation, modification,
     /// or access time of `path` according to `date_type`.
     /// 
@@ -189,29 +150,31 @@ impl Sorter {
         mytime
     }
 
-    /// Return the access date and time of `path` as the number of seconds since the
-    /// UNIX epoch.
+    /// Return the access date and time of `path` as the number of seconds since the epoch.
+    /// Now works cross-platform.
     fn get_epoch_secs_access(&self, path: &File) -> i64 {
-        let ctime_system = path.pathbuf.metadata().unwrap().accessed().expect("Failed to get atime");
-        let secs: i64 = ctime_system.duration_since(UNIX_EPOCH).unwrap().as_secs() as i64;
+        let metadata = path.pathbuf.metadata().unwrap();
+        let secs: i64 = FileTime::from_last_access_time(&metadata).seconds() as i64;
 
         secs
     }
     
-    /// Return the creation date and time of `path` as the number of seconds since the
-    /// UNIX epoch.
+    /// Return the creation date and time of `path` as the number of seconds since the epoch.
+    /// Now works cross-platform.
     fn get_epoch_secs_creation(&self, path: &File) -> i64 {
-        let ctime_system = path.pathbuf.metadata().unwrap().created().expect("Failed to get ctime");
-        let secs: i64 = ctime_system.duration_since(UNIX_EPOCH).unwrap().as_secs() as i64;
+        let metadata = path.pathbuf.metadata().unwrap();
+        let secs: i64 = FileTime::from_creation_time(&metadata).expect("Failed to get ctime.").seconds() as i64;
 
         secs
     }
 
-    /// Return the modification date and time of `path` as the number of seconds since the
-    /// UNIX epoch.
+    /// Return the modification date and time of `path` as the number of seconds since the epoch.
+    /// Now works cross-platform.
     fn get_epoch_secs_modified(&self, path: &File) -> i64 {
-        let ctime_system = path.pathbuf.metadata().unwrap().modified().expect("Failed to get mtime");
-        let secs: i64 = ctime_system.duration_since(UNIX_EPOCH).unwrap().as_secs() as i64;
+        let metadata = path.pathbuf.metadata().unwrap();
+        let secs: i64 = FileTime::from_last_modification_time(&metadata).seconds() as i64;
+        println!("secs: {} timestamp: {}", secs, 1641033122);
+        println!("{}", secs < 1641033122);
 
         secs
     }
@@ -287,7 +250,19 @@ impl Sorter {
         date_type: &str,
         preserve_name: &bool,
         exclude_type: (&str, bool),
-        only_type: (&str, bool)) -> (usize, Vec<File>, Vec<File>) {
+        only_type: (&str, bool)) -> Result<(usize, Vec<File>, Vec<File>), PathDoesNotExistError> {
+
+        // Return error messages if either source or target don't exist
+        if !source.exists() {
+            return Err(PathDoesNotExistError {
+                path: source.to_string(),
+            })
+        }
+        if !target.exists() {
+            return Err(PathDoesNotExistError {
+                path: target.to_string()
+            })
+        }
 
         // The vector to return: a tuple of (old_filename, new_filename)
         let mut vec_old: Vec<File> = Vec::new();
@@ -331,7 +306,7 @@ impl Sorter {
                 }
             }
         }
-        (items_to_sort, vec_old, vec_new)
+        Ok((items_to_sort, vec_old, vec_new))
     }
 
     /// Return [`true`] if:
@@ -413,6 +388,15 @@ impl Sorter {
             only_type
         );
 
+        match results {
+            Ok(ref result) => result,
+            Err(error) => {
+                println!("{}", error);
+                panic!("Sorting results error.");
+            },
+        };
+        let results = results.unwrap();
+
         // Sort the files, or dry run if specified
         if !dry_run {
 
@@ -427,4 +411,105 @@ impl Sorter {
         }
         (results.0, results.1, results.2)
     }
+}
+impl FromJson<File> for Sorter {
+
+    /// In the `SorteryLib` directory, there is a file called `template.json` This
+    /// should be used as a base for all JSON configuration files. Here is what
+    /// `template.json` looks like:
+    /// 
+    /// ```ignore
+    /// {
+    ///     "date_format": "%Y-%m-%d %Hh%Mm%Ss",
+    ///     "date_type": "m",
+    ///     "exclude_type": ["png"],
+    ///     "only_type": ["json", "py"],
+    ///     "preserve_name": false
+    /// }
+    /// ```
+    /// 
+    /// The items in the JSON file correspond with the fields in [`Sorter`] of the
+    /// same names.
+    /// 
+    /// Return a new [`Sorter`] instance, created using the configuration data in
+    /// a JSON [`File`] passed to the `json` argument. The source and target
+    /// directories must be passed to [`Sorter::from_json`] as well. For example:
+    /// 
+    /// ```ignore
+    /// use sorterylib::prelude::*;
+    /// use std::{fs, path::Path};
+    /// 
+    /// fn main() {
+    /// 
+    ///     // The path to the JSON file
+    ///     let json_file = File::from("test.json");
+    /// 
+    ///     // Create a Sorter instance based on the json file
+    ///     let sorter = Sorter::from_json(
+    ///         json_file,
+    ///         File::from("/path/to/source/dir/"), // The source directory
+    ///         File::from("/path/to/target/dir/") // The target directory
+    ///     );
+    /// }
+    /// ```
+    fn from_json(json: File, source: File, target: File) -> Sorter {
+
+        // Get the String from the file
+        let json_string = fs::read_to_string(json.to_path_buf()).expect("Failed to read JSON file.");
+
+        // Get the data from the JSON string
+        let data = ConfigData::from_json(&json_string);
+
+        Sorter {
+            source: source,
+            target: target,
+            date_format: data.date_format,
+            date_type: data.date_type,
+            preserve_name: data.preserve_name,
+            exclude_type: data.exclude_type,
+            only_type: data.only_type
+        }
+    }
+}
+impl FromJson<String> for Sorter {
+
+    /// Return a new [`Sorter`] instance, created using the configuration data in
+    /// a JSON [`String`] passed to the `json` argument. The source and target
+    /// directories must be passed to [`Sorter::from_json`] as well. For example:
+    /// 
+    /// ```ignore
+    /// use sorterylib::prelude::*;
+    /// use std::{fs, path::Path};
+    /// 
+    /// fn main() {
+    /// 
+    ///     // The path to the JSON file
+    ///     let json_path = Path::new("test.json");
+    /// 
+    ///     // Load the JSON string from the JSON file
+    ///     let json_string = fs::read_to_string(json_path).expect("Failed to read file");
+    /// 
+    ///     // Create a Sorter instance based on the json file
+    ///     let sorter = Sorter::from_json(
+    ///         json_string,
+    ///         File::from("/path/to/source/dir/"), // The source directory
+    ///         File::from("/path/to/target/dir/") // The target directory
+    ///     );
+    /// }
+    /// ```
+    fn from_json(json_string: String, source: File, target: File) -> Sorter {
+
+        // Get the data from the JSON string
+        let data = ConfigData::from_json(&json_string);
+
+        Sorter {
+            source: source,
+            target: target,
+            date_format: data.date_format,
+            date_type: data.date_type,
+            preserve_name: data.preserve_name,
+            exclude_type: data.exclude_type,
+            only_type: data.only_type
+        }
+    }
 }
\ No newline at end of file
